
#include "mySpi.h"
#include "display.h"



typedef struct display_s {
	int16_t old_value;
} display_t;

display_t displays[NUM_OF_DISP_UNITS];

uint8_t old_disable = 247; // illegal value

void dispInit() {

	spiXfer16(0x09ff); // decode mode
	spiXfer16(0x0a01); // intensity

	spiXfer16(0x0b07); // scan limit

	spiXfer16(0x0c01); // normal op

	spiXfer16(0x0f00); // led test: normal

	for (uint8_t i = 0; i < NUM_OF_DISP_UNITS; i++) {
		displays[i].old_value = 11111; // illegal value
	}
}

void dispDisable(uint8_t disable) {
	if (disable != old_disable) {
		old_disable = disable;
		if (disable) {
			spiXfer16(0x0c00);
		} else {
			spiXfer16(0x0c01);
		}
	}
}

void dispSetError() {
	spiXfer16(0x010f);
	spiXfer16(0x020f);
	spiXfer16(0x030f);
	spiXfer16(0x040b);
}

#define MAX_DIGIT 8

void dispSetDigit(uint8_t digit, uint8_t value) {
	if ((digit > MAX_DIGIT) || (digit == 0) || (value > 9)) {
		dispSetError();
	} else {
		uint16_t v = (((uint16_t) digit) << 8) | value;
		spiXfer16(v);
	}
}


void dispSetFloat(uint8_t unit, float value) {
	  if ((value < -99) || (value > 999)) {
	    dispSetError();
	  } else {
	    bool neg = value < 0;
			value = (value < 0) ? -1 * value : value;
	    uint16_t preComma = (uint16_t)value;
	    uint16_t postComma = round((value - (float)preComma) * 10);
	    if (postComma == 10) {
	      postComma = 0;
	      preComma += 1;
	    }
	    if ((preComma == 0) && (postComma == 0)) {
	      neg = false;
	    }
	
      uint8_t digit4 = postComma;
	
	    uint8_t digit3 = preComma / 100;
	    uint8_t remVal = preComma - (digit3 * 100);
	    uint8_t digit2 = remVal / 10;
	    remVal = remVal - (digit2 * 10);
	    uint8_t digit1 = remVal;
	
	    if (digit3 == 0) {
	      if (neg) {
	        digit3 = 0x0a;
	      } else {
	        digit3 = 0x0f;
	      }
	    }
	
	    if (digit3 > 0x09 && digit2 == 0) {
	      if (neg) {
	        digit2 = 0x0a;
	        digit3 = 0x0f;
	      } else {
	        digit2 = 0x0f;
	      }
	    }
	
	    // digit3 |= 0x80;
	
			if (unit == 0) {
				spiXfer16(0x0100 | digit1);
				spiXfer16(0x0200 | digit2);
				spiXfer16(0x0300 | digit3);
				spiXfer16(0x0400 | digit4);
			} else {
				spiXfer16(0x0500 | digit1);
				spiXfer16(0x0600 | digit2);
				spiXfer16(0x0700 | digit3);
				spiXfer16(0x0800 | digit4);
			}
	  }
}

void dispSet(uint8_t unit, int16_t value) {
	if (displays[unit].old_value != value) {
		displays[unit].old_value = value;

		if ((value < -999) || (value > 9999)) {
			dispSetError();
		} else {
			bool neg = value < 0;
			value = (value < 0) ? -1 * value : value;
			// value = abs(value);

			uint8_t digit3 = value / 1000;
			uint16_t remVal = value - (digit3 * 1000);
			uint8_t digit2 = remVal / 100;
			remVal = remVal - (digit2 * 100);
			uint8_t digit1 = remVal / 10;
			remVal = remVal - (digit1 * 10);
			uint8_t digit0 = remVal;

			if (digit3 == 0) {
				if (neg) {
					digit3 = 0x0a;
				} else {
					digit3 = 0x0f;
				}
			}

			if (digit3 > 0x09 && digit2 == 0) {
				if (neg) {
					digit2 = 0x0a;
					digit3 = 0x0f;
				} else {
					digit2 = 0x0f;
				}
			}

			if (digit3 > 0x09 && digit2 > 0x09 && digit1 == 0) {
				if (neg) {
					digit1 = 0x0a;
					digit2 = 0x0f;
				} else {
					digit1 = 0x0f;
				}
			}

			if (unit == 0) {
				spiXfer16(0x0100 | digit0);
				spiXfer16(0x0200 | digit1);
				spiXfer16(0x0300 | digit2);
				spiXfer16(0x0400 | digit3);
			} else {
				spiXfer16(0x0500 | digit0);
				spiXfer16(0x0600 | digit1);
				spiXfer16(0x0700 | digit2);
				spiXfer16(0x0800 | digit3);
			}
		}
	}
}



