#include "mySpi.h"
#include <intrinsics.h>
#include <msp430g2553.h>
#include <stdint.h>


const uint8_t SPI_CS = BIT1;
const uint8_t SPI_OUT = BIT0;
const uint8_t SPI_CLK = BIT2;

void spiInit() {
	P1DIR |= SPI_OUT | SPI_CS | SPI_CLK;
	P1OUT |= SPI_CS;
	P1OUT &= ~SPI_CLK;
}

void spiSetCS() {
	P1OUT &= ~SPI_CS;
}

void spiReleaseCS() {
	P1OUT |= SPI_CS;
	P1OUT &= ~SPI_CLK;
}

uint8_t spiXfer(uint8_t o) {
	for (int8_t i = 7; i >= 0; i--) {
		P1OUT &= ~SPI_CLK;
		uint8_t v = (o >> i) & 1;
		if (v == 1) {
			P1OUT |= SPI_OUT;
		} else {
			P1OUT &= ~SPI_OUT;
		}
		P1OUT |= SPI_CLK;
	}

	return 0;
}

uint8_t spiXfer16(uint16_t o) {
  spiSetCS();
  spiXfer((o >> 8) & 0x0ff);
  uint8_t res = 0;
  res = spiXfer(o & 0x0ff);
  spiReleaseCS();
  return res;
}

uint16_t spiXferD16(uint16_t o) {
  spiSetCS();
  uint16_t res = spiXfer((o >> 8) & 0x0ff);
  res <<= 8;
  res |= spiXfer(o && 0x0ff);
  spiReleaseCS();
  return res;
}

